<?php

namespace Drupal\default_content_access\Commands;

use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\default_content\Commands\DefaultContentCommands;
use Drupal\default_content\ExporterInterface;
use Drush\Attributes as CLI;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides Drush commands for the default_content_access module.
 *
 * @package Drupal\default_content_access
 */
class DefaultContentAccessCommands extends DefaultContentCommands {

  /**
   * Installed modules list from the 'container.modules' container parameter.
   *
   * @var string[]
   */
  protected array $installedExtensions;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * Commands constructor.
   *
   * @param \Drupal\default_content\ExporterInterface $default_content_exporter
   *   The default content exporter.
   * @param string[] $installed_modules
   *   Installed modules list from the 'container.modules' container parameter.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(
    ExporterInterface $default_content_exporter,
    array $installed_modules,
    ModuleHandlerInterface $module_handler,
    FileSystemInterface $file_system,
    Connection $database
  ) {
    parent::__construct($default_content_exporter);
    $this->installedExtensions = $installed_modules;
    $this->moduleHandler = $module_handler;
    $this->fileSystem = $file_system;
    $this->database = $database;
  }

  /**
   * Return an instance of these Drush commands.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container.
   *
   * @return \Drupal\default_content_access\Commands\DefaultContentAccessCommands
   *   The instance of Drush commands.
   */
  public static function create(ContainerInterface $container): DefaultContentAccessCommands {
    /** @var string[] $modules */
    $modules = $container->get('%container.modules%');
    return new DefaultContentAccessCommands(
      $container->get('default_content.exporter'),
      $modules,
      $container->get('module_handler'),
      $container->get('file_system'),
      $container->get('database'),
    );
  }

  /**
   * Exports all the content access setting defined in a module info file.
   *
   * @throws \JsonException
   */
  #[CLI\Command(name: 'default-content-access:export-module', aliases: ['dcaem'])]
  #[CLI\Argument('module', 'The name of the module.')]
  #[CLI\Usage(name: 'default-content-access:export-module module', description: 'Exports all the content access setting defined in a module info file.')]
  public function contentExportModule($module): void {
    if (!in_array($module, $this->installedExtensions, TRUE)) {
      return;
    }
    parent::contentExportModule($module);

    $serialized_by_type = $this->defaultContentExporter->exportModuleContent($module);
    $module_folder = $this->moduleHandler
      ->getModule($module)
      ->getPath() . '/access';
    $this->fileSystem->prepareDirectory($module_folder, FileSystemInterface::CREATE_DIRECTORY);

    $file_name = $module_folder . DIRECTORY_SEPARATOR . 'node.json';
    if (file_exists($file_name)) {
      $this->fileSystem->delete($file_name);
    }
    if (!empty($serialized_by_type['node'])) {
      $output = [];
      foreach ($serialized_by_type['node'] as $uuid => $value) {
        $query = $this->database->select('content_access', 'ca');
        $query->join('node', 'n', 'ca.nid = n.nid');
        $settings = $query->fields('ca', ['settings'])
          ->condition('n.uuid', $uuid)
          ->execute()
          ->fetchField();
        if (!empty($settings)) {
          $output[$uuid] = $settings;
        }
      }
      file_put_contents($file_name, json_encode($output, JSON_THROW_ON_ERROR));
    }
  }

  /**
   * Imports all the content access defined in a module info file.
   *
   * @throws \JsonException
   */
  #[CLI\Command(name: 'default-content-access:import-module', aliases: ['dcaim'])]
  #[CLI\Argument('module', 'The name of the module.')]
  #[CLI\Option('update-existing', 'Flag if existing content should be updated, defaults to FALSE.')]
  #[CLI\Usage(name: 'default-content-access:import-module module', description: 'Exports all the content access setting defined in a module info file.')]
  public function contentImportModule(string $module, array $options = ['update-existing' => FALSE]): void {
    if (!in_array($module, $this->installedExtensions, TRUE)) {
      return;
    }
    if (!method_exists(DefaultContentCommands::class, 'contentImportModule')) {
      // @todo This has not landed in the official code yet.
      // @see https://www.drupal.org/project/default_content/issues/2640734
      return;
    }
    // @phpstan-ignore-next-line
    parent::contentImportModule($module, $options);

    $serialized_by_type = $this->defaultContentExporter->exportModuleContent($module);
    $module_folder = $this->moduleHandler
      ->getModule($module)
      ->getPath() . '/access';
    $file_name = $module_folder . DIRECTORY_SEPARATOR . 'node.json';
    if (!file_exists($file_name)) {
      return;
    }
    $settings = json_decode(file_get_contents($file_name), TRUE, 512, JSON_THROW_ON_ERROR);
    $nids = $this->database->select('node', 'n')
      ->fields('n', ['uuid', 'nid'])
      ->condition('n.uuid', array_keys($serialized_by_type['node']), 'IN')
      ->execute()
      ->fetchAllKeyed();
    $this->database->delete('content_access')
      ->condition('nid', array_values($nids), 'IN')
      ->execute();
    $query = $this->database->insert('content_access')
      ->fields(['nid', 'settings']);
    foreach ($settings as $uuid => $setting) {
      $query->values([$nids[$uuid], $setting]);
    }
    try {
      $query->execute();
    }
    catch (\Exception) {
      // @todo Log this exception.
    }
    node_access_rebuild();
  }

}
